@@ -8,7 +8,7 @@ import pandas as pd
 __all__ = ["download", "biosample_to_dataframe"]
 
 
-def download(url: str, path: str = None, block_size: int = 32768):
+def download(url: str, path: str = None, block_size: int = 32768, cache: bool = False, append: bool = False):
     """Download file at given url showing a loading bar.
 
     Parameters
@@ -19,9 +19,20 @@ def download(url: str, path: str = None, block_size: int = 32768):
         The path where to store the data, if None the end of the url is used.
     block_size:int=1024,
         The download block size.
+    append: bool = False,
+        Wethever to append to the given file or not.
+    cache: bool = False,
+        Wethever to skip download if local file already exists.
+
+    Raises
+    --------------------------
+    ValueError,
+        If the request has not a status code 200 (success).
     """
     if path is None:
         path = url.split("/")[-1]
+    if cache and os.path.exists(path):
+        return
     r = requests.get(url, stream=True)
     total_size = int(r.headers.get('content-length', 0))
     t = tqdm(
@@ -38,7 +49,8 @@ def download(url: str, path: str = None, block_size: int = 32768):
     # If the user hits ctrl-c during the download we want to remove the partial
     # downloaded file.
     try:
-        with open(path, 'wb') as f:
+        modality = "ab" if append else "wb"
+        with open(path, modality) as f:
             for data in r.iter_content(block_size):
                 t.update(len(data))
                 f.write(data)
@@ -72,7 +84,17 @@ def sample_informations(sample: Dict) -> Dict:
 
     return {
         "organism": organism,
+        "accession": sample["accession"],
+        "status": sample["status"],
+        "assay_title": sample["assay_title"],
+        "assay_term_name": sample["assay_term_name"],
+        "replication_type": sample["replication_type"],
+        "date_released": sample["date_released"],
+        "date_created": sample["date_created"].split("T")[0],
+        "term_id": sample["biosample_ontology"]["term_id"],
         "cell_line": sample["biosample_ontology"]["term_name"],
+        "institute_name": sample["lab"]["institute_name"],
+        "title": sample["lab"]["title"],
         "target": label
     }
 
@@ -81,15 +103,31 @@ def sample_files_informations(sample: Dict) -> List[Dict]:
     """Return list of informations for every files in the sample.
         sample:Dict, the sample from which to extract the files informations.
     """
+    if len(sample["files"]) == 0:
+        return [{}]
+    keys = (
+        "accession",
+        "assay_term_name",
+        "assembly",
+        "file_format",
+        "file_size",
+        "file_type",
+        "output_category",
+        "output_type",
+        "read_length",
+        "read_length_units",
+        "run_type",
+        "schema_version",
+        "status",
+    )
     return [
         {
-            "status": f["status"] if "status" in f else None,
-            "accession":f["accession"] if "accession" in f else None,
-            "file_size":f["file_size"] if "file_size" in f else None,
-            "file_format":f["file_format"] if "file_format" in f else None,
-            "assembly":f["assembly"] if "assembly" in f else None,
-            "biological_replicates":sorted(f["biological_replicates"]),
-            "output_type":f["output_type"] if "output_type" in f else None,
+            **{
+                key: f[key] if key in f else None
+                for key in keys
+            },
+            "biological_replicates": sorted(f["biological_replicates"]) if "biological_replicates" in f else None,
+            "technical_replicates": sorted(f["technical_replicates"]) if "technical_replicates" in f else None,
             "url":f["cloud_metadata"]["url"] if "cloud_metadata" in f else None,
         } for f in sample["files"]
     ]
@@ -100,9 +138,15 @@ def biosample_to_dataframe(sample: Dict) -> pd.DataFrame:
         sample:Dict, the sample to convert into a simple DataFrame.
     """
     sample = normalize_sample(sample)
-    df = pd.DataFrame(sample_files_informations(sample))
-    for key, value in sample_informations(sample).items():
-        df[key] = value
+    data = sample_files_informations(sample)
+    metadata = sample_informations(sample)
+    df = pd.DataFrame([
+        {
+            **metadata,
+            **d,
+        }
+        for d in data
+    ])
     return df
 
 
